/* 
   Copyright (C) 2008 FreeIPMI Core Team
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.  
*/

#ifndef _IPMI_SENSORS_CONFIG_UTILS_H
#define _IPMI_SENSORS_CONFIG_UTILS_H

#include "ipmi-sensors-config.h"

config_err_t convert_id_string (ipmi_sensors_config_state_data_t *state_data, 
                                char *id_string);

config_err_t convert_event_string (ipmi_sensors_config_state_data_t *state_data, 
                                   char *event_string);

config_err_t create_section_name (ipmi_sensors_config_state_data_t *state_data, 
                                  uint8_t *sdr_record,
                                  unsigned int sdr_record_len,
                                  char *section_name,
                                  unsigned int section_name_len);

config_err_t get_sdr_record (ipmi_sensors_config_state_data_t *state_data,
                             const char *section_name,
                             uint8_t *sdr_record,
                             unsigned int *sdr_record_len);

#endif
